from authserv.test import *
from authserv.auth import authenticate
from authserv import protocol
from authserv.oath import totp


class AuthTest(unittest.TestCase):

    def test_main_password_ok(self):
        u = FakeUser('user', 'pass')
        self.assertEquals(
            protocol.OK,
            authenticate(u, 'svc', 'pass', None))

    def test_main_password_fail(self):
        u = FakeUser('user', 'pass')
        self.assertEquals(
            protocol.ERR_AUTHENTICATION_FAILURE,
            authenticate(u, 'svc', 'badpass', None))

    def test_require_otp(self):
        u = FakeUser('user', 'pass', otp_key=1234)
        self.assertEquals(
            protocol.ERR_OTP_REQUIRED,
            authenticate(u, 'svc', 'pass', None))
        self.assertEquals(
            protocol.ERR_OTP_REQUIRED,
            authenticate(u, 'svc', 'badpass', None))

    def test_otp_ok(self):
        secret = '089421'
        token = totp(secret, format='dec6', period=30)
        print 'otp is', token
        u = FakeUser('user', 'pass', otp_key=secret)
        self.assertEquals(
            protocol.OK,
            authenticate(u, 'svc', 'pass', str(token)))

    def test_otp_fail(self):
        u = FakeUser('user', 'pass', otp_key='123456')
        self.assertEquals(
            protocol.ERR_AUTHENTICATION_FAILURE,
            authenticate(u, 'svc', 'pass', '123456'))
        self.assertEquals(
            protocol.ERR_AUTHENTICATION_FAILURE,
            authenticate(u, 'svc', 'pass', 'malformed otp token!'))            

    def test_app_specific_password_ok(self):
        u = FakeUser('user', 'pass', asps=[
                ('svc', crypt.crypt('app-specific', 'zz'))])
        self.assertEquals(
            protocol.OK,
            authenticate(u, 'svc', 'app-specific', None))

    def test_app_specific_password_fail(self):
        u = FakeUser('user', 'pass', asps=[
                ('svc', crypt.crypt('app-specific', 'zz'))])
        self.assertEquals(
            protocol.ERR_AUTHENTICATION_FAILURE,
            authenticate(u, 'svc', 'badpass', None))
        self.assertEquals(
            protocol.ERR_AUTHENTICATION_FAILURE,
            authenticate(u, 'svc2', 'app-specific', None))
